// Datei: I2C.C
//
// I2C-Bus Schreib/Leseroutinen
//
// Holger Klabunde
// 27.04.2002
// Compiler CC5X

#include "prozess.h"      /* Der verwendete Prozessor */
#include "protos.h"

//I2C-Makro's
#define SCL	3
#define SDA	4

//#define SCL_ON() { TRISC.SCL=1; NOP; NOP; NOP; NOP; }
//#define SCL_OFF() { TRISC.SCL=0; NOP; NOP; NOP; NOP; }
//#define SDA_ON() { TRISC.SDA=1; NOP; NOP; NOP; NOP; }
//#define SDA_OFF() { TRISC.SDA=0; NOP; NOP; NOP; NOP; }

//I2C-Bus Adressen der angeschlossenen Bausteine
#define EEPROMWRITE  0xA0
#define EEPROMREAD   EEPROMWRITE+1

unsigned char I2C_Read();
void I2C_Write(unsigned char);

void SCL_ON() { TRISC.SCL=1; }
void SCL_OFF() { TRISC.SCL=0; }
void SDA_ON() { TRISC.SDA=1;  }
void SDA_OFF() { TRISC.SDA=0; }

void I2C_STOP() {  SCL_OFF(); SDA_OFF();  SCL_ON(); SDA_ON(); }
void I2C_START() {  I2C_STOP(); SDA_OFF(); SCL_OFF(); }

unsigned char ACKDATA;

//##################################
//Liest ein Zeichen aus dem EEPROM
//##################################
void ReadChar(unsigned char ch)
{
 unsigned char by,i;
 uns16 adr;

 adr=(uns16)ch * (uns16)5; //5 Bytes pro Zeichen
 adr++; //Der Font im EEPROM ist um ein Byte verrutscht. Warum ?

 ACKDATA=0;      //Sende Ack=0 fr mehrere Bytes empfangen
 
 I2C_START(); //Startbedingung senden,enthlt I2C_STOP()!!

 I2C_Write(EEPROMWRITE); //I2C-Adresse bertragen
 by=(unsigned char)(adr>>8);
 I2C_Write(by); //Zu lesende Adresse setzen
 by=(unsigned char)(adr&0xFF);
 I2C_Write(by); //Zu lesende Adresse setzen

 I2C_START(); //Neue Startbedingung senden,enthlt I2C_STOP()!!
 I2C_Write(EEPROMREAD); //I2C-Adresse mit LesenBit senden

 for(i=5; i>0; i--) //Hm Bytes verkehrt herum
  {

   if(i==1) ACKDATA=1;  //Kein ACK beim letzten Byte
   
   by=I2C_Read(); //Byte abholen
   charbuff[i-1]=by;
  }
  
 I2C_STOP(); //bertragung beendet
}

//***************************
//Liest ein Byte
//***************************
unsigned char I2C_Read(void)
{
 unsigned char i ,by ,mask ;

 by=0;  //Zum reinodern der Bits

 SDA_ON(); //Pullup SDA

 mask=0x80;  //D7 first
 for(i=0; i<8; i++)
  {
   SCL_ON();
   if(PORTC.SDA==1) by|=mask; //1 Bits reinodern
   SCL_OFF();
   mask>>=1;
  }

 if(ACKDATA) { SDA_ON(); } //Kein ACK 
 else        { SDA_OFF(); } //Sende ACK
 
 SCL_ON(); 
 SCL_OFF();

 return by;
}

//***************************
//Schreibt ein Byte bitweise
//***************************
void I2C_Write(unsigned char by)
{
 unsigned char i ,mask ;

 mask=0x80;

/* Byte senden mit SCL 0,1,0 */
/* D7,D6,...D1,D0 */
 for(i=0; i<8; i++)
  {
   if((by&mask)>0) { SDA_ON(); } /* eine 1 senden */
   else { SDA_OFF(); } /* sonst eine 0 */

   SCL_ON(); /* Flanke zum Daten bernehmen */
   SCL_OFF(); /* und fertig */
   mask=mask>>1;          /* nchstes Bit */
  }

/* Auf Acknowledge warten */
 SDA_ON(); /* Wird Eingang, Pullup zieht auf 1 */
 
 SCL_ON(); /* Ack raustakten */
// if(PORTC.SDA==1)
//  { I2CERR=1; } /* Kein ACK bei SDA=1 ! */
// else
//  { I2CERR=0; } //Alles klar

 SCL_OFF();

}
